# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.

from nevow import compy

from nevow.inevow import IGettable, ISettable, IContainer, IData
from nevow.stan import directive
from twisted.internet.defer import Deferred

def convertToData(data, context):
    newdata = IGettable(data, persist=False, default=None)
    if newdata is not None:
        newdata = newdata.get(context)
        if isinstance(newdata, Deferred):
            return newdata.addCallback(convertToData, context)
        else:
            return convertToData(newdata, context)
    else:
        return data

class NoAccessor(NotImplementedError):
    pass


class DirectiveAccessor(compy.Adapter):
    __implements__ = IGettable

    def get(self, context):
        data = context.locate(IData)
        container = IContainer(data, None, persist=False)
        if container is None:
            raise NoAccessor, "%s does not implement IContainer, and there is no registered adapter." % data
        child = container.child(context, self.original.name)
        return child

class FunctionAccessor(compy.Adapter):
    __implements__ = IGettable
    def get(self, context):
        return self.original(context, context.locate(IData))

class DictionaryContainer(compy.Adapter):
    __implements__ = IContainer
    
    def child(self, context, name):
        return self.original[name]

class ListContainer(compy.Adapter):
    __implements__ = IContainer

    def child(self, context, name):
        return self.original[int(name)]

