from __future__ import generators

import types

from twisted.internet import defer

from nevow import compy
from nevow import inevow
from nevow import tags


# NOTE:
# If you're about to import something from this module then you probably want
# to get it from nevow.flat instead. The idea is that flat's __init__.py may
# get smarter about the version of serialize/flatten/etc that is used,
# depending on what else is available. For instance, non-Twisted versions of
# these function may be imported into nevow.flat if Twisted is not installed.


# FIXME: hack!
# and the way serialize is implemented is possibly an
# abuse of the compy system, but hey, it WORKS. :)

cachedAdapters = {}
def getSerializer(obj):
    if hasattr(obj, '__class__'):
        klas = obj.__class__
    else:
        klas = type(obj)

    adapter = cachedAdapters.get(klas, None)
    if adapter is not None:
        return adapter
    
    #print "Adding cache entry for ",klas
    registry = compy.getRegistry(None)
    fromInterfaces = compy.classToInterfaces(klas)
    for fromInterface in fromInterfaces:
        # print " trying: ", fromInterface
        adapter = registry.getAdapterFactory(
            fromInterface,
            inevow.ISerializable,
            None)
        if adapter is not None:
            cachedAdapters[klas] = adapter
            return adapter
    return None


def serialize(obj, context):
    serializer = getSerializer(obj)
    if serializer is not None:
        return serializer(obj, context)

    raise NotImplementedError(
        '%r instance (type %s) does not implement %s, and '
        'there is no registered adapter.' %
        (obj, type(obj), inevow.ISerializable))


def iterflatten(stan, ctx, writer, shouldYieldItem=None):
    """This is the main meat of the nevow renderer. End-user programmers should
    instead use either flatten or precompile.
    """
    # 'rest' is a list of generators.
    # initialize as one-element list of a one-element generator of 
    rest = [ iter([serialize(stan, ctx)]) ]
    straccum = []
    while rest:
        gen = rest.pop()
        for item in gen:
            if isinstance(item, str):
                straccum.append(item)
            elif isinstance(item, unicode):
                straccum.append(item.encode('utf8'))
            elif isinstance(item, (list, types.GeneratorType)):
                # stop iterating this generator and put it back on the stack
                # and start iterating the new item instead.
                rest.append(gen)
                rest.append(iter(item))
                break
            else:
                if straccum:
                    writer(tags.xml(''.join(straccum)))
                    del straccum[:]
                if shouldYieldItem is not None and shouldYieldItem(item):
                    replacement = []
                    yield item, replacement.append
                    rest.append(gen)
                    rest.append(iter([replacement]))
                    break
                else:
                    if ctx.precompile:
                        ## We're precompiling and this is an item which can't be calculated until render time
                        ## add it to the list in 'precompile'
                        writer(item)
                    else:
                        rest.append(gen)
                        rest.append(iter([serialize(item, ctx)]))
                        break
    writer(tags.xml(''.join(straccum)))


def flatten(stan, ctx=None):
    """Given the stan and the optional context, return a string containing the
    representation of the tree in the given context.
    """
    if ctx is None:
        from nevow.context import RequestContext, PageContext
        ctx = RequestContext().with(PageContext())
    result = []
    list(iterflatten(stan, ctx, result.append))
    return ''.join(result)


def precompile(stan, ctx=None):
    """Given the stan and the optional context, return a list of strings and
    Context instances, optimizing as much static content as possible into contiguous
    string runs.

    The Context instances will have Tag instances whose .children have also been
    precompiled.
    """
    if ctx is None:
        from nevow.context import WovenContext
        ctx = WovenContext(precompile=True)
    doc = []
    list(iterflatten(stan, ctx, doc.append))
    return doc

