

## liveevil

from twisted.internet import reactor
from twisted.web import resource, server
from twisted.protocols import http
from nevow import events, tags
from nevow import inevow
from nevow import compy


def callJS(func, *args):
    """Return a Javascript string formatted such that the JS function named by the string 'func'
    will be called and the args specified as strings in *args will be quoted properly to prevent JS
    syntax errors.
    """
    args = ["'%s'" % a.replace("'", "\\'").replace('"', '\\"') for a in args]
    return "%s(%s);" % (func, ','.join(args))


class ILiveEvil(compy.Interface):
    def hookupOutput(self, output, finisher=None):
        """hook up an output conduit to this live evil instance.
        """

    def sendScript(self, script):
        """send a script through the output conduit to the browser.
        If no output conduit is yet hooked up, buffer the script
        until one is.
        """

    def handleInput(self, identifier, *args):
        """route some input from the browser to the appropriate
        destination.
        """


class LiveEvil(object):
    """An object which represents the client-side webbrowser.
    """
    __implements__ = ILiveEvil,

    output = None
    finisher = None
    def __init__(self, request, credentials):
        request.getSession().setComponent(ILiveEvil, self)
        # TODO set up an event hub
        self.events = events.EventNotification()
        self.hookupNotifications = []
        self.outputBuffer = []

    def hookupOutput(self, output, finisher = None):
        #print "output hooked up."
        self.output = output
        self.finisher = finisher
        for buf in self.outputBuffer:
            output(buf)
            #print "Sent buffered item: ", buf
        self.outputBuffer = []
        for notify in self.hookupNotifications:
            notify(self)

    def addNotification(self, notify):
        self.hookupNotifications.append(notify)

    def sendScript(self, script):
        if self.output:
            #print "output sent!", script
            self.output(script)
        else:
            self.outputBuffer.append(script)
            print "output buffered!", script

    def handleInput(self, identifier, *args):
        #self.sendScript(input)
        self.events.publish(identifier, *(self, ) + args)


class Output(resource.Resource):
    def render(self, request):
        self.mind = mind = request.getSession().getComponent(ILiveEvil)

        ## When the connection is lost, reset LiveEvil.output to None
        ## so we know when to buffer and when not to buffer
        def _resetOutput(results, liveevil):
            liveevil.output = None
            return results
        request.notifyFinish().addBoth(_resetOutput, mind)

        mind.hookupOutput(request.write, request.finish)
        reactor.callLater(30, self.noop)
        return server.NOT_DONE_YET

    def noop(self):
        if self.mind.output:
            #print "NOOP"
            self.mind.sendScript('null;')
            reactor.callLater(30, self.noop)


liveOutput = Output()


class Input(resource.Resource):
    def render(self, request):
        mind = request.getSession().getComponent(ILiveEvil)
        reactor.callLater(0, mind.handleInput, request.args['target'][0], *request.args.get('arguments',()))
        return "<html><body>done</body></html>"


liveInput = Input()


glue = tags.script(language="javascript")[
    tags.xml("""
offset = 0;
xmlhttp = new XMLHttpRequest();

xmlhttp.onreadystatechange = function() {
  if (xmlhttp.readyState == 3) {
    var content = xmlhttp.responseText.substring(offset, xmlhttp.responseText.length);
    eval(content);
    offset += content.length;
  } else if (xmlhttp.readyState == 4) {
        //alert(xmlhttp.responseText)
  }
}

xmlhttp.open("GET", "nevow_liveOutput", true)
xmlhttp.send(null)

function nevow_clientToServerEvent(theTarget) {
    var additionalArguments = ''
    for (i = 1; i<arguments.length; i++) {
        additionalArguments += '&arguments='
        additionalArguments += escape(eval(arguments[i]))
    }

    input = new XMLHttpRequest();
    input.onreadystatechange = function() {
        //alert('change');
    }
    input.open("GET", "nevow_liveInput?target=" + escape(theTarget)+additionalArguments)
    input.send(null)
}
    """)
]


i = 0
def next():
    global i
    i += 1
    return i


def handler(callable, *args):
    def block(context, data):
        identifier = "%x" % next()
        try:
            mind = context.locate(ILiveEvil)
        except KeyError:
            # Remove this when the request context looks in components and the session context is in place and also looks in components.
            mind = context.locate(inevow.IRequest).getSession().getComponent(ILiveEvil)
        mind.events.subscribe(identifier, callable)
        argstr = ','.join(["'%s'" % x.replace("'", "\\'") for x in args])
        if argstr:
            argstr = ','+argstr
        js = "nevow_clientToServerEvent('%(id)s'%(argstr)s); return false;" % {'id': identifier, 'argstr': argstr}
        #print "JS", js
        return js
    return block

