# This module is part of the Quotient project and is Copyright 2003 Divmod:
# http://www.divmod.org/.  This is free software.  You can redistribute it
# and/or modify it under the terms of version 2.1 of the GNU Lesser General
# Public License as published by the Free Software Foundation.


from nevow.tags import *
from nevow import testutil
from nevow.util import Deferred

from nevow import rend
from nevow import loaders


def deferit(data):
    return data.d

def deferdot(data):
    return data.d2

class LaterRenderTest(testutil.TestCase):
    
    def setUp(self):
        self.d = Deferred()
        self.d2 = Deferred()
        self.r = rend.Page(
            docFactory=loaders.stan(
                html(data=self)[
                    'Hello ', invisible[invisible[invisible[invisible[deferit]]]],
                    deferdot,
                    ]
                )
            )

    def test_deferredSupport(self):
        req = testutil.FakeRequest()
        self.r.renderHTTP(req)
        self.assertEquals(req.v, '<html>Hello ')
        self.d.callback("world")
        self.assertEquals(req.v, '<html>Hello world')
        self.d2.callback(".")
        self.assertEquals(req.v, '<html>Hello world.</html>')


    def test_deferredSupport2(self):
        req = testutil.FakeRequest()
        self.r.renderHTTP(req)
        self.assertEquals(req.v, '<html>Hello ')
        self.d2.callback(".")
        self.assertEquals(req.v, '<html>Hello ')
        self.d.callback("world")
        self.assertEquals(req.v, '<html>Hello world.</html>')

    def test_deferredSupport3(self):
        req = testutil.FakeRequest()
        self.r.buffered = True
        self.r.renderHTTP(req)
        self.assertEquals(req.v, '')
        self.d.callback("world")
        self.assertEquals(req.v, '')
        self.d2.callback(".")
        self.assertEquals(req.v, '<html>Hello world.</html>')

class LaterDataTest(testutil.TestCase):
    def data_later(self, context, data):
        return self.d

    def data_later2(self, context, data):
        return self.d2


    def setUp(self):
        self.d = Deferred()
        self.d2 = Deferred()
        self.r = rend.Page(docFactory=loaders.stan(
            html(data=self.data_later)[
                'Hello ', str, ' and '
                'goodbye ',str,
                span(data=self.data_later2, render=str)]))

    def test_deferredSupport(self):
        req = testutil.FakeRequest()
        self.r.renderHTTP(req)
        self.assertEquals(req.v, '')
        self.d.callback("world")
        self.assertEquals(req.v, '<html>Hello world and goodbye world')
        self.d2.callback(".")
        self.assertEquals(req.v, '<html>Hello world and goodbye world.</html>')

