
import time
from datetime import date
import calendar
calendar.setfirstweekday(calendar.SUNDAY)

from nevow import rend


class ScheduleRoot(rend.Page):
    docFactory = rend.htmlfile('Month.html')

    def __init__(self, calendarEntries):
        self.calendarEntries = calendarEntries
        super(ScheduleRoot, self).__init__()

    def data_currentMonth(self, context, data):
        curtime = time.localtime()
        year = int(context.arg('year', curtime[0]))
        month = int(context.arg('month', curtime[1]))
        return year, month, calendar.monthcalendar(year, month)

    def render_month(self, context, data):
        year, month, weeksAndDays = data

        weekPattern = context.patternGenerator('calendarWeek')
        dayPattern = context.with(weekPattern()).patternGenerator('calendarDay')

        calendarBody = []
        for week in weeksAndDays:
            currentWeek = weekPattern().clear()
            calendarBody.append(currentWeek)
            for day in week:
                if day != 0:
                    events = self.calendarEntries.eventsFor(date(year, month, day))
                else:
                    events = []
                currentDay = dayPattern(render=self.render_day, data=(year, month, day, events))
                currentWeek.children.append(currentDay)

        context.fillSlots('label', "%s %s" % (calendar.month_name[month], year))
        context.fillSlots('calendarBody', calendarBody)
        return context.tag

    def render_day(self, context, data):
        year, month, day, events = data
        if events:
            from nevow.url import here
            # Construct URL to child page
            url = here.child(str(year)).child(str(month)).child(str(day))
            eventsDOM = context.onePattern('events').clear()
            eventsDOM.attributes['href'] =url
            eventsDOM.children.append(
                "%s event%s" % (len(events), len(events) > 1 and 's' or ''))
        else:
            eventsDOM = ''

        context.fillSlots('date', day)
        context.fillSlots('events', eventsDOM)
        return context.tag

    def data_yearsAndMonths(self, context, data):
        return [[(month, year) for month in range(1, 13)]
            for year in range(2003, 2006)]

    def render_monthSelector(self, context, data):
        tag = context.tag.clear()
        tag.attributes['href'] = "?month=%s&year=%s" % data
        tag.children.append("%s %s" % (calendar.month_name[data[0]], data[1]))
        return tag